;------------PIC_SWRR.ASM-------------------------------
;PIC_SWR.ASM, revised to add Morse SWR readout
;For auto adjusting SWR meter using 16C71 by Bert Kelley 
;2050 tones
	list	p=16c71
	__config  0x3ff1
        radix	hex
        errorlevel -302
;-----------------------
indf	equ	0x00    ;part of indirect addr
pc	equ	0x02    ;program counter
porta	equ	0x05    ;
portb	equ	0x06    ;
status	equ	0x03    ; 
rp0	equ	5       ;bank switch bit
c	equ	0       ;carry flag    
z	equ	2       ;z flag bit
w	equ	0       ;w register
f	equ	1       ;file
fsr	equ	0x04    ;indirect pointer
adcon0	equ	0x08    ;a/d control register
adres	equ	0x09    ;holds result a to d
intcon	equ	0x0b    ;not used         
trisa	equ	0x85    ;bank 1 porta
trisb	equ	0x86    ;bank 1 portb
adcon1	equ	0x88    ;a/d control register
;variables
addrsav	equ	0x0c    ;
count	equ	0x0d    ;utilty counter  
ncount	equ	0x0e    ;holds 1 millisec delay 
mcount	equ	0x0f    ;delay
temp    equ     0x10    ;chkpwr, Morse count
fwd     equ     0x11    ;forward power from A/D
rev     equ     0x12    ;reverse power from A/D
fprlo   equ	0x13    ;lsb result of add F + R
fprhi   equ     0x14    ;msb result of add F + R         
fminusr equ     0x15    ;result of fwd minus rev
swrlo   equ	0x16    ;F + R/F - R remainder    
swrhi   equ     0x17    ;F + R/F - R quot./dec Msd              
prodlo  equ     0x18    ;product low from multiply    
prodhi  equ     0x19    ;product high from multiply
dendhi  equ     0x1a    ;dividend high in division 
dendlo  equ     0x1b    ;dividend low in division       
dvsrlo  equ	0x1c    ;divisor in division call
tenths  equ     0x1d    ;holds swr tenths in bcd
tone    equ     0x1e    ;freq of Morse tone
dsplsd  equ     0x24    ;7 seg swr tenths to display
dspmsd  equ	0x25    ;7 seg swr units to display
cw_1    equ     0x26    ;first cw character
cw_2    equ     0x27    ;second cw character
cw_3    equ     0x28    ;last cw character
shift   equ     0x29    ;cw character shift reg.
;--------------------------
	org	0x000
;
start   bsf	status,rp0 ;switch to bank 1
        movlw   0x0b       ;configure porta 0,1,3 in;  
	movwf	trisa      ;porta 0,4 out           
	movlw	0x00       ;portb set all outputs
	movwf	trisb
	movlw	0x02       ;Port A bits 0,1 a/d inputs
	movwf	adcon1     ;bits 2,3,4 digital
	bcf	status,rp0 ;switch back to bank0
	clrf	portb      ;clear output port
	bcf	intcon,7   ;disable interrupt
        bsf     porta,3    ;set porta, bit 3, high
                           ;this has PB on it
        bcf     porta,4    ;clear porta, bit 4, low
                           ;has piezo spkr on it
;-------FORWARD POWER ACQUISITION--
getfwd   		  
      	movlw	0x81       ;config A/D, select AN0
	movwf	adcon0     ;
	call	del_20     ;wait 20 microseconds
	bsf	adcon0,2   ;start conversion
test	btfsc	adcon0,2   ;test for go/done bit
	goto	test       ;
	movf	adres,w    ;get conversion,
        movwf	fwd        ;                           
 	goto    chkpwr     ;
;-------CHECK INPUT POWER--
chkpwr  movf	fwd,0     ;                    
        movwf	temp      ;       
      	movlw	0x20	  ;
  	subwf	temp,f    ;
 	btfsc	status,0  ;                          
	goto    getrev    ;
	movlw	0x38      ;seven seg for "L"
        movwf	dspmsd    ;to table
	movlw   0x48      ;morse for "L"
        movwf   cw_1      ;
        movlw   0x73      ;seven seg for "P"
        movwf   dsplsd    ;to table
        movlw   0x68      ;morse for "P"
        movwf   cw_2      ;
        movlw   0x02      ;
        movwf   fprhi     ;morse counter
        goto    exit      ;
;-------REVERSE POWER ACQUISTION------
getrev  movlw	0x89      ;turn on AN1
	movwf	adcon0    ;
 	call	del_20    ;
	bsf	adcon0,2  ;
testagn btfsc	adcon0,2  ;
	goto    testagn   ;
	movf	adres,w   ; 
	movwf	rev       ;reverse stored
        goto	add_sub   ;
;-------PROCESS FORWARD & REVERSE------
add_sub clrf	fprhi     ;add F + R, (Fwd + Rev)-->fprhi,
	clrf	fprlo     ;fprlo.  (Fwd - Rev)-->fminus r.
        movf	rev,w     ;                                       
	movwf	fprlo	  ;
        movf	fwd,w     ;                           
	addwf	fprlo,f   ;
	btfsc	status,c  ;                                 
	incf	fprhi,f   ;F+R-->fprhi,fprlo
        movf    rev,w     ;Subtract Fwd - Rev-->fminusr
	subwf	fwd,w     ;                                 
        movwf   fminusr   ;setup for division to follow
 	movwf	dvsrlo    ;
	movf	fprhi,w   ;
        movwf   dendhi    ;
        movf    fprlo,w   ;
        movwf   dendlo    ;
                          ;
	call	divide    ;(F+R) divided by (F-R)            
	movf	dendlo,w  ;Quotient in dendlo, rem. in dendhi
	movwf   swrhi     ;swrhi in bcd if < 10 decimal
	movlw	0x0a      ;check if swr is 10 or more
        subwf	swrhi,w   ;
	btfsc	status,c  ;
	goto    hi_swr    ;is over
        movf    swrhi,w   ;is less than 10
	call    swr2cw    ;convert msd to morse
        movwf   cw_1      ;load it as first character
        movlw   0x56      ;morse decimal pt
        movwf   cw_2      ;store it as second character
        clrw              ;
	movf	dendhi,w  ; 
	movwf	swrlo     ;
        movlw   0x03      ;Load morse count
        movwf   fprhi     ;
        goto    convert   ;
;-------CONVERT HI & LO SWR FOR DISPLAY---
convert	movlw	0x0a      ;converts tenths swr to bcd    
 	movwf	count     ;multiply swrlo by 0ah =
        clrf    prodhi    ;  --> prodhi and prodlo
 	clrf	prodlo    ;multiply by adding,
 	movf    swrlo,w   ;remainder is added 0a
mpy   	addwf	prodlo,f  ;times.                               
 	btfsc   status,c  ;
        goto    inc_hi
tally	decfsz	count,f   ;decrement multiplier
 	goto	mpy         
        goto    bcd       ;                                                             
        ;
inc_hi	incf	prodhi,f  ;count carrys
 	goto	tally
        ;
bcd   	movf	prodhi,w  ;divides prodhi,lo              
	movwf   dendhi    ;by fminusr
        movf    prodlo,w  ;
        movwf   dendlo    ;
        movf    fminusr,w ;
        movwf   dvsrlo    ;
        call    divide    ;                                 
	movf    dendlo,w  ;dendlo in divide holds 
        movwf   tenths    ;tenths
        call    swr2cw    ;convert tenths swr to morse
        movwf   cw_3      ;load it as 3rd character
        clrw              ;
	movf	swrhi,w	  ;                                
    	call    segmnt    ;convert to 7 segment
	movwf   dspmsd    ;
        bsf     dspmsd,7  ;set decimal point
	clrf	dsplsd    ;
        movf	tenths,w  ;
   	call    segmnt    ;
        movwf   dsplsd    ;
exit                      ;                              
        call    display   ; 
	btfss	porta,3   ;CW pushbutton down?
        goto    sendcw    ;
	goto    getfwd    ;
;
hi_swr  movlw	0x76      ;H to
 	movwf   dspmsd    ;display MSD
        movlw   0x08      ;morse H
        movwf   cw_1      ;
        movlw   0x20      ;morse I    
        movwf   cw_2      ;
        movlw   0x06      ;I  (HI for high SWR)
        movwf   dsplsd    ;to display LSD
        movlw   0x02      ;
        movwf   fprhi     ;# morse characters
        goto    exit      ;
                          ;
;-Divide program written by Bob Fehrenbach, from
;http://members.tripod,com/~mdileo/snippic.html 
;
divide  movlw   0x08
	movwf	count  
	clrf	temp      ;temp holds borrow
divloop rlf	dendlo,f  ;16 bit lsd's in dendlo
	rlf	dendhi,f  ;16 bit msd's in dendhi
	rlf	temp,f    
	movf	dvsrlo,w  ;8 bit divisor
	subwf	dendhi,w
	btfss	temp,0    
	skpnc	
	movwf	dendhi
	btfsc	temp,0   
	rrf	temp,f
	decfsz	count,f
	goto	divloop   ;quotient in dendlo
	rlf	dendlo,f  ;remainder in dendhi
	return
;
;-------------------------sendcw
sendcw  call	del_3e    ;button debounce wait
	btfss   porta,3   ;send after button released
        goto    sendcw    ;
	movlw   0x26      ;
        movwf   fsr       ;set fsr to start CW message
	movf    indf,w    ;first char to shift register
        movwf   shift     ;
rotate  bcf     status,c  ;clear carry flag
        rlf	shift,1   ;rotate shift register left
        btfsc   status,c  ;skip next if Cy not set
        goto    dah       ;send a dah if flag
        goto    dit       ;send a dit if no flag
chkeoc  movf	shift,w   ;check if end of character
	sublw   0x80      ;                          
        btfss   status,z  ;
	goto    rotate    ;              
        decfsz  fprhi,1   ;   
        goto    getmor    ;
        goto    getfwd    ;done, resume processing SWR
getmor  call    del_1e    ;total space between characters 
        call    del_1e    ;is three elements including
        incf    fsr,1     ;one el. appended to dot's/dash's
        movf    indf,w    ; 
        movwf   shift      ;next char. to send register
        goto    rotate    ;
;------------------------ call segmnt
segmnt	addwf	pc,f     ;Convert bcd for LED      
	retlw	0x3f     ;0
	retlw	0x06     ;1
	retlw	0x5b     ;2
	retlw	0x4f	 ;3
	retlw	0x66	 ;4
	retlw	0x6d     ;5
	retlw	0x7d     ;6
	retlw	0x07	 ;7
	retlw	0x7f	 ;8
	retlw	0x6f	 ;9
	return
;           
swr2cw  addwf   pc,f     ;Conversion 
        retlw   0xfc     ;0
        retlw   0x7c     ;1
        retlw   0x3c     ;2
        retlw   0x1c     ;3
        retlw   0x0c     ;4
        retlw   0x04     ;5
        retlw   0x84     ;6
        retlw   0xc4     ;7
        retlw   0xe4     ;8
        retlw   0xf4     ;9 
	return
;-----------------------------CALL DEL__20
del_20  movlw	0x10
        movwf	count
again 	decfsz	count,f
	goto	again 
 	return
;-----------------------------CALL WAIT
wait	movlw   0xf4
        movwf   ncount    ;1 millisec delay
delay1  decf    ncount,1  ;
        btfss   status,2
        goto	delay1
        return
;-----------------------------1E DELAY
del_1e  movlw   0x3c      ;60 millisecond delay             
	movwf	mcount    ;                                                       
main	call	wait      ;
	decfsz  mcount,1
	goto    main
	return
;
	org	0x100
;
;----------drives two segment SWR display
display movlw	0x24      ;               
        movwf	fsr       ;set fsr to start  
        movf    indf,0    ;get data
	movwf	portb     ;
	bcf     porta,2
	call    wait      ;output lsb     
  	bsf	porta,2   ;
        incf    fsr,1     ;
        movf    indf,0
	movwf   portb
        bsf     porta,2
        call    wait
        movlw   0x00	  ;exits with all
	movwf   portb     ;digits off
        bcf     porta,2
        return
;-----------------------------3E DELAY
del_3e  movlw	0x03      ;180 milliseconds
	movwf	count     ;                   
reload  movlw   0x3c      ;
        movwf   mcount    ;
circle  call    wait
	decfsz  mcount,1 
	goto    circle
	decfsz  count,1 
	goto    reload
        return
;---------------------------- Dit
dit	movlw   0x78      ;Morse dot   
        movwf   mcount    ;Frequency of tones is 2055 hz
uphalf  bsf	porta,4   ;to match piezo speaker response                           
	movlw   0x50      ;length of one half 
	movwf   tone	  ;cycle is in 'tone.'
upper   decfsz  tone,f    ;
        goto    upper     ;
	bcf     porta,4   ;start half of cycle 
	movlw   0x50      ;dot is 60 milliseconds long plus
	movwf   tone      ;another 60 milliseconds silence
lower   decfsz  tone,f    ;(20 wpm)
	goto    lower     ;
	decfsz  mcount,f  ;
	goto	uphalf    ;
        movlw   0x3c      ;1T silence after dots
        movwf   temp      ;60 millisec
rest    call    wait      ;
        decfsz  temp,f    ;
        goto    rest      ;
 	goto    chkeoc    ;
;---------------------------- Dah
dah	movlw   0x03      ;Morse dash
        movwf   count     ;is 3X length of a dot      
restor  movlw   0x78      ;or 180 milliseconds
        movwf   mcount    ;plus 60 ms silence
hicy    bsf	porta,4   ;upper half of cycle 
	movlw   0x50      ;begins                    
	movwf   tone	  ;
hitime  decfsz  tone,f    ;
        goto    hitime    ;
	bcf     porta,4   ;lower half of cycle 
	movlw   0x50      ;begins
	movwf   tone      ;
lotime  decfsz  tone,f    ;
	goto    lotime    ;
	decfsz  mcount,f  ;
	goto	hicy      ;
	decfsz  count,1   ;
        goto    restor    ;
	movlw	0x3c      ;1T silence
        movwf	temp      ;after dash
quiet   call	wait
	decfsz  temp,f
	goto    quiet     ;
        goto    chkeoc
;
	end
; 